@@ -1,3 +1,4 @@
+import re
 import os
 import sys
 import ssl
@@ -17,7 +18,7 @@ from six.moves.socketserver import ThreadingMixIn
 from six.moves.urllib.parse import urlparse
 from six.moves.BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
 from localstack.config import TMP_FOLDER, USE_SSL, EXTRA_CORS_ALLOWED_HEADERS, EXTRA_CORS_EXPOSE_HEADERS
-from localstack.constants import ENV_INTERNAL_TEST_RUN
+from localstack.constants import ENV_INTERNAL_TEST_RUN, APPLICATION_JSON
 from localstack.utils.common import FuncThread, generate_ssl_cert, to_bytes
 
 QUIET = False
@@ -26,15 +27,15 @@ QUIET = False
 SERVER_CERT_PEM_FILE = '%s/server.test.pem' % (TMP_FOLDER)
 
 
-CORS_ALLOWED_HEADERS = ('authorization', 'content-type', 'content-md5', 'cache-control',
+CORS_ALLOWED_HEADERS = ['authorization', 'content-type', 'content-md5', 'cache-control',
     'x-amz-content-sha256', 'x-amz-date', 'x-amz-security-token', 'x-amz-user-agent',
-    'x-amz-acl', 'x-amz-version-id')
+    'x-amz-target', 'x-amz-acl', 'x-amz-version-id', 'x-localstack-target', 'x-amz-tagging']
 if EXTRA_CORS_ALLOWED_HEADERS:
-    CORS_ALLOWED_HEADERS += tuple(EXTRA_CORS_ALLOWED_HEADERS.split(','))
+    CORS_ALLOWED_HEADERS += EXTRA_CORS_ALLOWED_HEADERS.split(',')
 
 CORS_ALLOWED_METHODS = ('HEAD', 'GET', 'PUT', 'POST', 'DELETE', 'OPTIONS', 'PATCH')
 
-CORS_EXPOSE_HEADERS = ('x-amz-version-id',)
+CORS_EXPOSE_HEADERS = ('x-amz-version-id', )
 if EXTRA_CORS_EXPOSE_HEADERS:
     CORS_EXPOSE_HEADERS += tuple(EXTRA_CORS_EXPOSE_HEADERS.split(','))
 
@@ -66,7 +67,7 @@ class ProxyListener(object):
         """
         return True
 
-    def return_response(self, method, path, data, headers, response):
+    def return_response(self, method, path, data, headers, response, request_handler=None):
         """ This interceptor method is called by the proxy when returning a response
             (*after* having forwarded the request and received a response from the backend
             service). It receives details of the incoming request as well as the response
@@ -79,6 +80,12 @@ class ProxyListener(object):
         """
         return None
 
+    def get_forward_url(self, method, path, data, headers):
+        """ Return a custom URL to forward the given request to. If a falsy value is returned,
+            then the default URL will be used.
+        """
+        return None
+
 
 class GenericProxyHandler(BaseHTTPRequestHandler):
 
@@ -96,6 +103,9 @@ class GenericProxyHandler(BaseHTTPRequestHandler):
             BaseHTTPRequestHandler.__init__(self, request, client_address, server)
         except SSLError as e:
             LOG.warning('SSL error when handling request: %s' % e)
+        except Exception as e:
+            if 'cannot read from timed out object' not in str(e):
+                LOG.warning('Unknown error: %s' % e)
 
     def parse_request(self):
         result = BaseHTTPRequestHandler.parse_request(self)
@@ -150,6 +160,11 @@ class GenericProxyHandler(BaseHTTPRequestHandler):
         self.method = requests.options
         self.forward('OPTIONS')
 
+    def do_CONNECT(self):
+        self.method = None
+        self.headers['Connection'] = self.headers.get('Connection') or 'keep-alive'
+        self.forward('CONNECT')
+
     def read_content(self):
         content_length = self.headers.get('Content-Length')
         if content_length:
@@ -190,33 +205,43 @@ class GenericProxyHandler(BaseHTTPRequestHandler):
         return ', '.join(x_forwarded_for_list)
 
     def forward(self, method):
-        path = self.path
-        if '://' in path:
-            path = '/' + path.split('://', 1)[1].split('/', 1)[1]
-        proxy_url = '%s%s' % (self.proxy.forward_url, path)
-        target_url = self.path
-        if '://' not in target_url:
-            target_url = '%s%s' % (self.proxy.forward_url, target_url)
         data = self.data_bytes
-
         forward_headers = CaseInsensitiveDict(self.headers)
+
+        # force close connection
+        if forward_headers.get('Connection') not in ['keep-alive', None]:
+            self.close_connection = 1
+
+        def is_full_url(url):
+            return re.match(r'[a-zA-Z]+://.+', url)
+
+        path = self.path
+        if is_full_url(path):
+            path = path.split('://', 1)[1]
+            path = '/%s' % (path.split('/', 1)[1] if '/' in path else '')
+        forward_base_url = self.proxy.forward_base_url
+        proxy_url = '%s%s' % (forward_base_url, path)
+
+        for listener in self._listeners():
+            if listener:
+                proxy_url = listener.get_forward_url(method, path, data, forward_headers) or proxy_url
+
+        target_url = self.path
+        if not is_full_url(target_url):
+            target_url = '%s%s' % (forward_base_url, target_url)
+
         # update original "Host" header (moto s3 relies on this behavior)
         if not forward_headers.get('Host'):
             forward_headers['host'] = urlparse(target_url).netloc
         if 'localhost.atlassian.io' in forward_headers.get('Host'):
             forward_headers['host'] = 'localhost'
-
         forward_headers['X-Forwarded-For'] = self.build_x_forwarded_for(forward_headers)
 
-        # force close connection
-        if forward_headers.get('Connection', '').lower() != 'keep-alive':
-            self.close_connection = 1
-
         try:
             response = None
             modified_request = None
             # update listener (pre-invocation)
-            for listener in self.DEFAULT_LISTENERS + [self.proxy.update_listener]:
+            for listener in self._listeners():
                 if not listener:
                     continue
                 listener_result = listener.forward_request(method=method,
@@ -227,6 +252,7 @@ class GenericProxyHandler(BaseHTTPRequestHandler):
                 if isinstance(listener_result, dict):
                     response = Response()
                     response._content = json.dumps(listener_result)
+                    response.headers['Content-Type'] = APPLICATION_JSON
                     response.status_code = 200
                     break
                 elif isinstance(listener_result, Request):
@@ -239,8 +265,11 @@ class GenericProxyHandler(BaseHTTPRequestHandler):
                     code = listener_result if isinstance(listener_result, int) else 503
                     self.send_response(code)
                     self.send_header('Content-Length', '0')
+                    # allow pre-flight CORS headers by default
+                    self._send_cors_headers()
                     self.end_headers()
                     return
+
             # perform the actual invocation of the backend service
             if response is None:
                 forward_headers['Connection'] = forward_headers.get('Connection') or 'close'
@@ -248,7 +277,7 @@ class GenericProxyHandler(BaseHTTPRequestHandler):
                 request_url = proxy_url
                 if modified_request:
                     if modified_request.url:
-                        request_url = '%s%s' % (self.proxy.forward_url, modified_request.url)
+                        request_url = '%s%s' % (forward_base_url, modified_request.url)
                     data_to_send = modified_request.data
 
                 response = self.method(request_url, data=data_to_send,
@@ -257,12 +286,13 @@ class GenericProxyHandler(BaseHTTPRequestHandler):
                 # prevent requests from processing response body
                 if not response._content_consumed and response.raw:
                     response._content = response.raw.read()
+
             # update listener (post-invocation)
             if self.proxy.update_listener:
                 kwargs = {
                     'method': method,
                     'path': path,
-                    'data': data,
+                    'data': self.data_bytes,
                     'headers': forward_headers,
                     'response': response
                 }
@@ -287,14 +317,7 @@ class GenericProxyHandler(BaseHTTPRequestHandler):
                 self.send_header('Content-Length', '%s' % len(response.content) if response.content else 0)
 
             # allow pre-flight CORS headers by default
-            if 'Access-Control-Allow-Origin' not in response.headers:
-                self.send_header('Access-Control-Allow-Origin', '*')
-            if 'Access-Control-Allow-Methods' not in response.headers:
-                self.send_header('Access-Control-Allow-Methods', ','.join(CORS_ALLOWED_METHODS))
-            if 'Access-Control-Allow-Headers' not in response.headers:
-                self.send_header('Access-Control-Allow-Headers', ','.join(CORS_ALLOWED_HEADERS))
-            if 'Access-Control-Expose-Headers' not in response.headers:
-                self.send_header('Access-Control-Expose-Headers', ','.join(CORS_EXPOSE_HEADERS))
+            self._send_cors_headers(response)
 
             self.end_headers()
             if response.content and len(response.content):
@@ -302,7 +325,8 @@ class GenericProxyHandler(BaseHTTPRequestHandler):
         except Exception as e:
             trace = str(traceback.format_exc())
             conn_errors = ('ConnectionRefusedError', 'NewConnectionError',
-                           'Connection aborted', 'Unexpected EOF', 'Connection reset by peer')
+                           'Connection aborted', 'Unexpected EOF', 'Connection reset by peer',
+                           'cannot read from timed out object')
             conn_error = any(e in trace for e in conn_errors)
             error_msg = 'Error forwarding request: %s %s' % (e, trace)
             if 'Broken pipe' in trace:
@@ -324,10 +348,49 @@ class GenericProxyHandler(BaseHTTPRequestHandler):
             except Exception as e:
                 LOG.warning('Unable to flush write file: %s' % e)
 
+    def _send_cors_headers(self, response=None):
+        # Note: Use "response is not None" here instead of "not response"!
+        headers = response is not None and response.headers or {}
+        if 'Access-Control-Allow-Origin' not in headers:
+            self.send_header('Access-Control-Allow-Origin', '*')
+        if 'Access-Control-Allow-Methods' not in headers:
+            self.send_header('Access-Control-Allow-Methods', ','.join(CORS_ALLOWED_METHODS))
+        if 'Access-Control-Allow-Headers' not in headers:
+            requested_headers = self.headers.get('Access-Control-Request-Headers', '')
+            requested_headers = re.split(r'[,\s]+', requested_headers) + CORS_ALLOWED_HEADERS
+            self.send_header('Access-Control-Allow-Headers', ','.join([h for h in requested_headers if h]))
+        if 'Access-Control-Expose-Headers' not in headers:
+            self.send_header('Access-Control-Expose-Headers', ','.join(CORS_EXPOSE_HEADERS))
+
+    def _listeners(self):
+        return self.DEFAULT_LISTENERS + [self.proxy.update_listener]
+
     def log_message(self, format, *args):
         return
 
 
+class DuplexSocket(ssl.SSLSocket):
+    """ Simple duplex socket wrapper that allows serving HTTP/HTTPS over the same port. """
+
+    def accept(self):
+        newsock, addr = socket.socket.accept(self)
+        peek_bytes = 5
+        first_bytes = newsock.recv(peek_bytes, socket.MSG_PEEK)
+        if len(first_bytes or '') == peek_bytes:
+            first_byte = first_bytes[0]
+            if first_byte < 32 or first_byte >= 127:
+                newsock = self.context.wrap_socket(newsock,
+                            do_handshake_on_connect=self.do_handshake_on_connect,
+                            suppress_ragged_eofs=self.suppress_ragged_eofs,
+                            server_side=True)
+
+        return newsock, addr
+
+
+# set globally defined SSL socket implementation class
+ssl.SSLContext.sslsocket_class = DuplexSocket
+
+
 class GenericProxy(FuncThread):
     def __init__(self, port, forward_url=None, ssl=False, host=None, update_listener=None, quiet=False, params={}):
         FuncThread.__init__(self, self.run_cmd, params, quiet=quiet)
@@ -339,7 +402,7 @@ class GenericProxy(FuncThread):
             if '://' not in forward_url:
                 forward_url = 'http://%s' % forward_url
             forward_url = forward_url.rstrip('/')
-        self.forward_url = forward_url
+        self.forward_base_url = forward_url
         self.update_listener = update_listener
         self.server_stopped = False
         # Required to enable 'Connection: keep-alive' for S3 uploads
@@ -351,7 +414,7 @@ class GenericProxy(FuncThread):
             self.httpd = ThreadedHTTPServer((self.listen_host, self.port), GenericProxyHandler)
             if self.ssl:
                 # make sure we have a cert generated
-                combined_file, cert_file_name, key_file_name = GenericProxy.create_ssl_cert()
+                combined_file, cert_file_name, key_file_name = GenericProxy.create_ssl_cert(serial_number=self.port)
                 self.httpd.socket = ssl.wrap_socket(self.httpd.socket,
                     server_side=True, certfile=combined_file)
             self.httpd.my_object = self
@@ -367,13 +430,13 @@ class GenericProxy(FuncThread):
             self.server_stopped = True
 
     @classmethod
-    def create_ssl_cert(cls, random=True):
-        return generate_ssl_cert(SERVER_CERT_PEM_FILE, random=random)
+    def create_ssl_cert(cls, serial_number=None):
+        return generate_ssl_cert(SERVER_CERT_PEM_FILE, serial_number=serial_number)
 
     @classmethod
-    def get_flask_ssl_context(cls):
+    def get_flask_ssl_context(cls, serial_number=None):
         if USE_SSL:
-            combined_file, cert_file_name, key_file_name = cls.create_ssl_cert()
+            combined_file, cert_file_name, key_file_name = cls.create_ssl_cert(serial_number=serial_number)
             return (cert_file_name, key_file_name)
         return None
 
@@ -385,7 +448,7 @@ def serve_flask_app(app, port, quiet=True, host=None, cors=True):
         logging.getLogger('werkzeug').setLevel(logging.ERROR)
     if not host:
         host = '0.0.0.0'
-    ssl_context = GenericProxy.get_flask_ssl_context()
+    ssl_context = GenericProxy.get_flask_ssl_context(serial_number=port)
     app.config['ENV'] = 'development'
 
     def noecho(*args, **kwargs):
