@@ -11,17 +11,23 @@ from localstack.services.install import ROOT_PATH
 LOGGER = logging.getLogger(__name__)
 
 
-def start_kinesis(port=None, asynchronous=False, shard_limit=100, update_listener=None):
+def start_kinesis(port=None, asynchronous=False, update_listener=None):
     port = port or config.PORT_KINESIS
     install.install_kinesalite()
     backend_port = DEFAULT_PORT_KINESIS_BACKEND
+    latency = config.KINESIS_LATENCY
     kinesis_data_dir_param = ''
     if config.DATA_DIR:
         kinesis_data_dir = '%s/kinesis' % config.DATA_DIR
         mkdir(kinesis_data_dir)
         kinesis_data_dir_param = '--path %s' % kinesis_data_dir
-    cmd = ('%s/node_modules/kinesalite/cli.js --shardLimit %s --port %s %s' %
-        (ROOT_PATH, shard_limit, backend_port, kinesis_data_dir_param))
+    cmd = (
+        '%s/node_modules/kinesalite/cli.js --shardLimit %s --port %s'
+        ' --createStreamMs %s --deleteStreamMs %s --updateStreamMs %s %s'
+    ) % (
+        ROOT_PATH, config.KINESIS_SHARD_LIMIT, backend_port,
+        latency, latency, latency, kinesis_data_dir_param
+    )
     print('Starting mock Kinesis (%s port %s)...' % (get_service_protocol(), port))
     start_proxy_for_service('kinesis', port, backend_port, update_listener)
     return do_run(cmd, asynchronous)
