@@ -6,14 +6,13 @@ import boto3
 import base64
 import logging
 import six
+from six.moves.urllib.parse import quote_plus, unquote_plus
 from localstack import config
 from localstack.constants import (
-    REGION_LOCAL, DEFAULT_REGION, LOCALHOST, MOTO_ACCOUNT_ID,
-    ENV_DEV, APPLICATION_AMZ_JSON_1_1, APPLICATION_AMZ_JSON_1_0,
-    APPLICATION_X_WWW_FORM_URLENCODED, TEST_AWS_ACCOUNT_ID)
+    REGION_LOCAL, LOCALHOST, MOTO_ACCOUNT_ID, ENV_DEV, APPLICATION_AMZ_JSON_1_1,
+    APPLICATION_AMZ_JSON_1_0, APPLICATION_X_WWW_FORM_URLENCODED, TEST_AWS_ACCOUNT_ID)
 from localstack.utils.common import (
-    run_safe, to_str, is_string, is_string_or_bytes, make_http_request,
-    timestamp, is_port_open, get_service_protocol)
+    run_safe, to_str, is_string, is_string_or_bytes, make_http_request, is_port_open, get_service_protocol)
 from localstack.utils.aws.aws_models import KinesisStream
 
 # AWS environment variable names
@@ -43,6 +42,9 @@ BOTO_CLIENTS_CACHE = {}
 # Assume role loop seconds
 DEFAULT_TIMER_LOOP_SECONDS = 60 * 50
 
+# maps SQS queue ARNs to queue URLs
+SQS_ARN_TO_URL_CACHE = {}
+
 
 class Environment(object):
     def __init__(self, region=None, prefix=None):
@@ -121,11 +123,20 @@ def is_local_env(env):
     return not env or env.region == REGION_LOCAL or env.prefix == ENV_DEV
 
 
-def connect_to_resource(service_name, env=None, region_name=None, endpoint_url=None):
-    """
-    Generic method to obtain an AWS service resource using boto3, based on environment, region, or custom endpoint_url.
-    """
-    return connect_to_service(service_name, client=False, env=env, region_name=region_name, endpoint_url=endpoint_url)
+class Boto3Session(boto3.session.Session):
+    """ Custom boto3 session that points to local endpoint URLs. """
+
+    def resource(self, service, *args, **kwargs):
+        self._fix_endpoint(kwargs)
+        return connect_to_resource(service, *args, **kwargs)
+
+    def client(self, service, *args, **kwargs):
+        self._fix_endpoint(kwargs)
+        return connect_to_service(service, *args, **kwargs)
+
+    def _fix_endpoint(self, kwargs):
+        if 'amazonaws.com' in kwargs.get('endpoint_url', ''):
+            kwargs.pop('endpoint_url')
 
 
 def get_boto3_credentials():
@@ -146,12 +157,17 @@ def get_boto3_session():
     return boto3
 
 
+def get_region():
+    # TODO look up region from context
+    return get_local_region()
+
+
 def get_local_region():
     global LOCAL_REGION
     if LOCAL_REGION is None:
         session = boto3.session.Session()
         LOCAL_REGION = session.region_name or ''
-    return LOCAL_REGION or DEFAULT_REGION
+    return LOCAL_REGION or config.DEFAULT_REGION
 
 
 def get_local_service_url(service_name_or_port):
@@ -161,7 +177,10 @@ def get_local_service_url(service_name_or_port):
     service_name = service_name_or_port
     if service_name == 's3api':
         service_name = 's3'
-    return os.environ['TEST_%s_URL' % (service_name.upper().replace('-', '_'))]
+    elif service_name == 'runtime.sagemaker':
+        service_name = 'sagemaker-runtime'
+    service_name_upper = service_name.upper().replace('-', '_').replace('.', '_')
+    return os.environ['TEST_%s_URL' % service_name_upper]
 
 
 def is_service_enabled(service_name):
@@ -174,15 +193,25 @@ def is_service_enabled(service_name):
         return False
 
 
-def connect_to_service(service_name, client=True, env=None, region_name=None, endpoint_url=None, config=None):
+def connect_to_resource(service_name, env=None, region_name=None, endpoint_url=None, *args, **kwargs):
+    """
+    Generic method to obtain an AWS service resource using boto3, based on environment, region, or custom endpoint_url.
+    """
+    return connect_to_service(service_name, client=False, env=env, region_name=region_name, endpoint_url=endpoint_url)
+
+
+def connect_to_service(service_name, client=True, env=None, region_name=None, endpoint_url=None,
+        config=None, *args, **kwargs):
     """
     Generic method to obtain an AWS service client using boto3, based on environment, region, or custom endpoint_url.
     """
-    key_elements = [service_name, client, env, region_name, endpoint_url, config]
+    region_name = region_name or get_region()
+    env = get_environment(env, region_name=region_name)
+    region = env.region if env.region != REGION_LOCAL else region_name
+    key_elements = [service_name, client, env, region, endpoint_url, config]
     cache_key = '/'.join([str(k) for k in key_elements])
     if cache_key not in BOTO_CLIENTS_CACHE:
         # Cache clients, as this is a relatively expensive operation
-        env = get_environment(env, region_name=region_name)
         my_session = get_boto3_session()
         method = my_session.client if client else my_session.resource
         verify = True
@@ -190,7 +219,6 @@ def connect_to_service(service_name, client=True, env=None, region_name=None, en
             if is_local_env(env):
                 endpoint_url = get_local_service_url(service_name)
                 verify = False
-        region = env.region if env.region != REGION_LOCAL else get_local_region()
         BOTO_CLIENTS_CACHE[cache_key] = method(service_name, region_name=region,
             endpoint_url=endpoint_url, verify=verify, config=config)
 
@@ -200,6 +228,7 @@ def connect_to_service(service_name, client=True, env=None, region_name=None, en
 class VelocityInput:
     """Simple class to mimick the behavior of variable '$input' in AWS API Gateway integration velocity templates.
     See: http://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-mapping-template-reference.html"""
+
     def __init__(self, value):
         self.value = value
 
@@ -214,10 +243,14 @@ class VelocityInput:
     def json(self, path):
         return json.dumps(self.path(path))
 
+    def __repr__(self):
+        return '$input'
+
 
 class VelocityUtil:
     """Simple class to mimick the behavior of variable '$util' in AWS API Gateway integration velocity templates.
     See: http://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-mapping-template-reference.html"""
+
     def base64Encode(self, s):
         if not isinstance(s, str):
             s = json.dumps(s)
@@ -230,15 +263,32 @@ class VelocityUtil:
             s = json.dumps(s)
         return base64.b64decode(s)
 
+    def toJson(self, obj):
+        return obj and json.dumps(obj)
 
-def render_velocity_template(template, context, as_json=False):
+    def urlEncode(self, s):
+        return quote_plus(s)
+
+    def urlDecode(self, s):
+        return unquote_plus(s)
+
+    def escapeJavaScript(self, s):
+        return str(s).replace("'", r"\'")
+
+
+def render_velocity_template(template, context, variables={}, as_json=False):
     import airspeed
+
+    # run a few fixes to properly prepare the template
+    template = re.sub(r'(^|\n)#\s+set(.*)', r'\1#set\2', template, re.MULTILINE)
+
     t = airspeed.Template(template)
-    variables = {
+    var_map = {
         'input': VelocityInput(context),
         'util': VelocityUtil()
     }
-    replaced = t.merge(variables)
+    var_map.update(variables or {})
+    replaced = t.merge(var_map)
     if as_json:
         replaced = json.loads(replaced)
     return replaced
@@ -260,9 +310,17 @@ def check_valid_region(headers):
         raise Exception('Invalid region specified in "Authorization" header: "%s"' % region)
 
 
+def set_default_region_in_headers(headers):
+    auth_header = headers.get('Authorization')
+    if not auth_header:
+        return
+    replaced = re.sub(r'(.*Credential=[^/]+/[^/]+/)([^/])+/', r'\1%s/' % get_region(), auth_header)
+    headers['Authorization'] = replaced
+
+
 def fix_account_id_in_arns(response, colon_delimiter=':', existing=None, replace=None):
     """ Fix the account ID in the ARNs returned in the given Flask response or string """
-    existing = existing or ['123456789', MOTO_ACCOUNT_ID]
+    existing = existing or ['123456789', '1234567890', MOTO_ACCOUNT_ID]
     existing = existing if isinstance(existing, list) else [existing]
     replace = replace or TEST_AWS_ACCOUNT_ID
     is_str_obj = is_string_or_bytes(response)
@@ -283,15 +341,26 @@ def fix_account_id_in_arns(response, colon_delimiter=':', existing=None, replace
 def get_s3_client():
     return boto3.resource('s3',
         endpoint_url=config.TEST_S3_URL,
-        config=boto3.session.Config(
-            s3={'addressing_style': 'path'}),
+        config=boto3.session.Config(s3={'addressing_style': 'path'}),
         verify=False)
 
 
+def sqs_queue_url_for_arn(queue_arn):
+    if '://' in queue_arn:
+        return queue_arn
+    if queue_arn in SQS_ARN_TO_URL_CACHE:
+        return SQS_ARN_TO_URL_CACHE[queue_arn]
+    sqs_client = connect_to_service('sqs')
+    parts = queue_arn.split(':')
+    result = sqs_client.get_queue_url(QueueName=parts[5], QueueOwnerAWSAccountId=parts[4])['QueueUrl']
+    SQS_ARN_TO_URL_CACHE[queue_arn] = result
+    return result
+
+
 def extract_region_from_auth_header(headers):
     auth = headers.get('Authorization') or ''
     region = re.sub(r'.*Credential=[^/]+/[^/]+/([^/]+)/.*', r'\1', auth)
-    region = region or get_local_region()
+    region = region or get_region()
     return region
 
 
@@ -331,26 +400,53 @@ def get_iam_role(resource, env=None):
     return 'role-%s' % resource
 
 
-def dynamodb_table_arn(table_name, account_id=None):
-    account_id = get_account_id(account_id)
-    return 'arn:aws:dynamodb:%s:%s:table/%s' % (get_local_region(), account_id, table_name)
+def secretsmanager_secret_arn(secret_name, account_id=None, region_name=None):
+    pattern = 'arn:aws:secretsmanager:%s:%s:secret:%s'
+    return _resource_arn(secret_name, pattern, account_id=account_id, region_name=region_name)
 
 
-def dynamodb_stream_arn(table_name, account_id=None):
+def cloudformation_stack_arn(stack_name, account_id=None, region_name=None):
+    pattern = 'arn:aws:cloudformation:%s:%s:stack/%s/id-1234'
+    return _resource_arn(stack_name, pattern, account_id=account_id, region_name=region_name)
+
+
+def dynamodb_table_arn(table_name, account_id=None, region_name=None):
+    pattern = 'arn:aws:dynamodb:%s:%s:table/%s'
+    return _resource_arn(table_name, pattern, account_id=account_id, region_name=region_name)
+
+
+def dynamodb_stream_arn(table_name, latest_stream_label, account_id=None):
     account_id = get_account_id(account_id)
     return ('arn:aws:dynamodb:%s:%s:table/%s/stream/%s' %
-        (get_local_region(), account_id, table_name, timestamp()))
+        (get_region(), account_id, table_name, latest_stream_label))
 
 
-def lambda_function_arn(function_name, account_id=None):
+def log_group_arn(group_name, account_id=None, region_name=None):
+    pattern = 'arn:aws:logs:%s:%s:log-group:%s'
+    return _resource_arn(group_name, pattern, account_id=account_id, region_name=region_name)
+
+
+def lambda_function_arn(function_name, account_id=None, region_name=None):
+    return lambda_function_or_layer_arn('function', function_name, account_id=account_id, region_name=region_name)
+
+
+def lambda_layer_arn(layer_name, version=None, account_id=None):
+    return lambda_function_or_layer_arn('layer', layer_name, version=None, account_id=account_id)
+
+
+def lambda_function_or_layer_arn(type, entity_name, version=None, account_id=None, region_name=None):
     pattern = 'arn:aws:lambda:.*:.*:(function|layer):.*'
-    if re.match(pattern, function_name):
-        return function_name
-    if ':' in function_name:
-        raise Exception('Lambda function name should not contain a colon ":"')
+    if re.match(pattern, entity_name):
+        return entity_name
+    if ':' in entity_name:
+        raise Exception('Lambda %s name should not contain a colon ":": %s' % (type, entity_name))
     account_id = get_account_id(account_id)
-    pattern = re.sub(r'\([^\|]+\|.+\)', 'function', pattern)
-    return pattern.replace('.*', '%s') % (get_local_region(), account_id, function_name)
+    region_name = region_name or get_region()
+    pattern = re.sub(r'\([^\|]+\|.+\)', type, pattern)
+    result = pattern.replace('.*', '%s') % (region_name, account_id, entity_name)
+    if version:
+        result = '%s:%s' % (result, version)
+    return result
 
 
 def lambda_function_name(name_or_arn):
@@ -361,42 +457,54 @@ def lambda_function_name(name_or_arn):
     return parts[6]
 
 
-def state_machine_arn(name, account_id=None):
-    if ':' in name:
-        return name
-    account_id = get_account_id(account_id)
+def state_machine_arn(name, account_id=None, region_name=None):
     pattern = 'arn:aws:states:%s:%s:stateMachine:%s'
-    return pattern % (get_local_region(), account_id, name)
+    return _resource_arn(name, pattern, account_id=account_id, region_name=region_name)
+
+
+def stepfunctions_activity_arn(name, account_id=None, region_name=None):
+    pattern = 'arn:aws:states:%s:%s:activity:%s'
+    return _resource_arn(name, pattern, account_id=account_id, region_name=region_name)
 
 
 def fix_arn(arn):
     """ Function that attempts to "canonicalize" the given ARN. This includes converting
         resource names to ARNs, replacing incorrect regions, account IDs, etc. """
     if arn.startswith('arn:aws:lambda'):
-        return lambda_function_arn(lambda_function_name(arn))
+        parts = arn.split(':')
+        region = parts[3] if parts[3] in config.VALID_REGIONS else get_region()
+        return lambda_function_arn(lambda_function_name(arn), region_name=region)
     LOG.warning('Unable to fix/canonicalize ARN: %s' % arn)
     return arn
 
 
-def cognito_user_pool_arn(user_pool_id, account_id=None):
-    account_id = get_account_id(account_id)
-    return 'arn:aws:cognito-idp:%s:%s:userpool/%s' % (get_local_region(), account_id, user_pool_id)
+def cognito_user_pool_arn(user_pool_id, account_id=None, region_name=None):
+    pattern = 'arn:aws:cognito-idp:%s:%s:userpool/%s'
+    return _resource_arn(user_pool_id, pattern, account_id=account_id, region_name=region_name)
 
 
 def kinesis_stream_arn(stream_name, account_id=None):
     account_id = get_account_id(account_id)
-    return 'arn:aws:kinesis:%s:%s:stream/%s' % (get_local_region(), account_id, stream_name)
+    return 'arn:aws:kinesis:%s:%s:stream/%s' % (get_region(), account_id, stream_name)
 
 
 def firehose_stream_arn(stream_name, account_id=None):
     account_id = get_account_id(account_id)
-    return ('arn:aws:firehose:%s:%s:deliverystream/%s' % (get_local_region(), account_id, stream_name))
+    return ('arn:aws:firehose:%s:%s:deliverystream/%s' % (get_region(), account_id, stream_name))
 
 
 def s3_bucket_arn(bucket_name, account_id=None):
     return 'arn:aws:s3:::%s' % (bucket_name)
 
 
+def _resource_arn(name, pattern, account_id=None, region_name=None):
+    if ':' in name:
+        return name
+    account_id = get_account_id(account_id)
+    region_name = region_name or get_region()
+    return pattern % (region_name, account_id, name)
+
+
 def create_sqs_queue(queue_name, env=None):
     env = get_environment(env)
     # queue
@@ -406,10 +514,16 @@ def create_sqs_queue(queue_name, env=None):
 
 def sqs_queue_arn(queue_name, account_id=None, region_name=None):
     account_id = get_account_id(account_id)
-    region_name = region_name or DEFAULT_REGION
+    region_name = region_name or get_region()
     return ('arn:aws:sqs:%s:%s:%s' % (region_name, account_id, queue_name))
 
 
+def apigateway_restapi_arn(api_id, account_id=None, region_name=None):
+    account_id = get_account_id(account_id)
+    region_name = region_name or get_region()
+    return ('arn:aws:apigateway:%s:%s:/restapis/%s' % (region_name, account_id, api_id))
+
+
 def sqs_queue_name(queue_arn):
     parts = queue_arn.split(':')
     return queue_arn if len(parts) == 1 else parts[5]
@@ -417,7 +531,7 @@ def sqs_queue_name(queue_arn):
 
 def sns_topic_arn(topic_name, account_id=None):
     account_id = get_account_id(account_id)
-    return ('arn:aws:sns:%s:%s:%s' % (get_local_region(), account_id, topic_name))
+    return ('arn:aws:sns:%s:%s:%s' % (get_region(), account_id, topic_name))
 
 
 def get_sqs_queue_url(queue_arn):
@@ -443,7 +557,7 @@ def mock_aws_request_headers(service='dynamodb', region_name=None):
     elif service == 'sqs':
         ctype = APPLICATION_X_WWW_FORM_URLENCODED
     access_key = get_boto3_credentials().access_key
-    region_name = region_name or DEFAULT_REGION
+    region_name = region_name or get_region()
     headers = {
         'Content-Type': ctype,
         'Accept-Encoding': 'identity',
@@ -636,7 +750,8 @@ def create_api_gateway_integrations(api_id, resource_id, method,
 
 
 def apigateway_invocations_arn(lambda_uri):
-    return 'arn:aws:apigateway:%s:lambda:path/2015-03-31/functions/%s/invocations' % (DEFAULT_REGION, lambda_uri)
+    return ('arn:aws:apigateway:%s:lambda:path/2015-03-31/functions/%s/invocations' %
+        (get_region(), lambda_uri))
 
 
 def get_elasticsearch_endpoint(domain=None, region_name=None):
